@@ -1,20 +1,21 @@
-"""
-Custom widgets
-"""
-try:
-    import json
-except ImportError:
-    import simplejson as json
+# -*- coding: utf-8 -*-
 
 from flask import current_app, Markup
 from werkzeug import url_encode
-
-# use flaskext.babel for translations, if available
+from flask.json import dumps, JSONEncoder
+from .._compat import text_type
 
 try:
-    from flaskext.babel import gettext as _
-except ImportError:
-    _ = lambda(s): s
+    from speaklater import _LazyString
+
+    class _JSONEncoder(JSONEncoder):
+        def default(self, o):
+            if isinstance(o, _LazyString):
+                return str(o)
+            return JSONEncoder.default(self, o)
+except:
+    _JSONEncoder = JSONEncoder
+
 
 RECAPTCHA_API_SERVER = 'http://api.recaptcha.net/'
 RECAPTCHA_SSL_API_SERVER = 'https://www.google.com/recaptcha/api/'
@@ -23,8 +24,11 @@ RECAPTCHA_HTML = u'''
 <script type="text/javascript" src="%(script_url)s"></script>
 <noscript>
   <div><iframe src="%(frame_url)s" height="300" width="500"></iframe></div>
-  <div><textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
-  <input type="hidden" name="recaptcha_response_field" value="manual_challenge"></div>
+  <div>
+    <textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
+    <input type="hidden" name="recaptcha_response_field"
+           value="manual_challenge" />
+  </div>
 </noscript>
 '''
 
@@ -34,21 +38,19 @@ __all__ = ["RecaptchaWidget"]
 class RecaptchaWidget(object):
 
     def recaptcha_html(self, server, query, options):
-        return Markup(RECAPTCHA_HTML % dict(
+        html = current_app.config.get('RECAPTCHA_HTML', RECAPTCHA_HTML)
+        return Markup(html % dict(
             script_url='%schallenge?%s' % (server, query),
             frame_url='%snoscript?%s' % (server, query),
-            options=json.dumps(options)
+            options=dumps(options, cls=_JSONEncoder)
         ))
 
     def __call__(self, field, error=None, **kwargs):
         """Returns the recaptcha input HTML."""
 
         if current_app.config.get('RECAPTCHA_USE_SSL', False):
-
             server = RECAPTCHA_SSL_API_SERVER
-
         else:
-
             server = RECAPTCHA_API_SERVER
 
         try:
@@ -58,21 +60,23 @@ class RecaptchaWidget(object):
         query_options = dict(k=public_key)
 
         if field.recaptcha_error is not None:
-            query_options['error'] = unicode(field.recaptcha_error)
+            query_options['error'] = text_type(field.recaptcha_error)
 
         query = url_encode(query_options)
 
+        _ = field.gettext
+
         options = {
-           'theme': 'clean',
+            'theme': 'clean',
             'custom_translations': {
-                'visual_challenge':    _('Get a visual challenge'),
-                'audio_challenge':     _('Get an audio challenge'),
-                'refresh_btn':         _('Get a new challenge'),
+                'visual_challenge': _('Get a visual challenge'),
+                'audio_challenge': _('Get an audio challenge'),
+                'refresh_btn': _('Get a new challenge'),
                 'instructions_visual': _('Type the two words:'),
-                'instructions_audio':  _('Type what you hear:'),
-                'help_btn':            _('Help'),
-                'play_again':          _('Play sound again'),
-                'cant_hear_this':      _('Download sound as MP3'),
+                'instructions_audio': _('Type what you hear:'),
+                'help_btn': _('Help'),
+                'play_again': _('Play sound again'),
+                'cant_hear_this': _('Download sound as MP3'),
                 'incorrect_try_again': _('Incorrect. Try again.'),
             }
         }
